"""
/**********************************************************************
FILE     : $RCSfile: MenuItem.py,v $
PURPOSE  : MenuItem Class and MenuFolder sample Object
NOTES    : may be considered as 'stable' 
AUTHOR   : Simon Eisenmann
COPYRIGHT: (c) 2001 by struktur AG
DATE     : 14FEB2002
REVISION : $Revision: 1.30 $
VERSION  : $Id: MenuItem.py,v 1.30 2002/07/24 09:38:49 longsleep Exp $ (Author: $Author: longsleep $)

struktur AG            Phone: +49 711 8966560
Junghansstr. 5         Fax:   +49 711 89665610
70469 Stuttgart        email: info@struktur.de
GERMANY 

http://www.struktur.de
http://www.strukturag.com

**********************************************************************/

"""

__doc__ = 'MenuItem'
__version__ = '0.0.1'

from OFS.Folder import Folder
from Globals import DTMLFile
from AccessControl import getSecurityManager
from Products.ZCatalog.ZCatalog import ZCatalog
from Products.ZCatalog.CatalogPathAwareness import CatalogAware
from Products.ZCatalog import CatalogBrains
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Persistence import Persistent
from OFS.Cache import Cacheable
from AccessControl.PermissionRole import rolesForPermissionOn

from urllib import quote
from string import join, split
from Acquisition import aq_base
import ExtensionClass, Globals, types, os, time


# helpers
def url2path(url):
    path = tuple(split(url,'/'))
    if (len(path) > 1 ) and (path[-1] == ''): path = path[:-1]
    return path

def path2url(path, ending_slash=1):
    url = join(path,'/')
    if url=='': 
        url="/"
    elif ending_slash:
        url = url + '/'
    return url

def sort4weight(a, b):
    
    # sort
    if callable(a.getMenuWeight):
        x=a.getMenuWeight()
    else:
        x=a.getMenuWeight

    if callable(b.getMenuWeight):
        y=b.getMenuWeight()
    else:
        y=b.getMenuWeight

    return cmp(x,y)


class MenuSorter:
    '''
    helper class for sorting
    '''
    customSortMethod = None

    
    def __init__(self, customSortMethod=None):
        ''' inits the helper class '''

        if callable(customSortMethod):
            self.customSortMethod=customSortMethod


    def sort4all(self, a, b):
        ''' sorts 4 customsort -> weight -> ABC '''
        
        result=0

        # customsort
        if self.customSortMethod:
            try:
                result=self.customSortMethod(a, b)
            except:
                print "error in custom sort method"

        if not result:
            # sort 4 weight
            #print "weight"
            
            result=sort4weight(a, b)

        if not result:
            # sort 4 title
            #print "title"
            result=cmp(a.title, b.title)

        if not result:
            # sort 4 id
            #print "id"
            result=cmp(a.id, b.id)

        #print "sorted", a.id, b.id, result
        return result


    def sortMenuLevel(self, brainList):
        ''' sorts '''

        # TODO: implement lazy sorting
        # TODO: discover how much performance diff is from lazyMap to list
        # NOTE: brainList is a lazyMap which does not support sorting -> convert to list

        if type(brainList) != types.ListType:
            brainList=list(brainList)
        brainList.sort(self.sort4all)
        return brainList


class MenuBrain:
    '''
    catalog brain
    '''
    
    def isSelected(self, selectedMenuURL):
        ''' returns 1 or 0 if this brain is part of the selected menu or not '''

        selectedMenuPath=url2path(selectedMenuURL)

        if self.getMenuPath == selectedMenuPath[:len(self.getMenuPath)]:
            return 1
        else:
            return 0

    def getVirtualURL(self):
        #print "dudu"
        # This is useful for physical path relative to a VirtualRoot
        try:
            req = self.REQUEST
        except AttributeError:
            #print "no request"
            req = {}
        rpp = req.get('VirtualRootPhysicalPath', ('',))
        spp = self.getPhysicalPath
        i = 0
        #print "rpp, spp", rpp, spp
        for name in rpp[:len(spp)]:
            #print "name", name, spp[i]
            if spp[i] == name:
                i = i + 1
            else:
                break
            
        path = map(quote, spp[i:])
        #print "path", path, spp[i:]
        path = path2url(path)
        if path[0] != '/':
            path='/'+path
        
        return path
            

class MenuCatalog(ZCatalog):
    '''
    catalog for the menu
    '''
    _v_brains = MenuBrain

    def __init__(self, id, title='', vocab_id=None, container=None):

        # call init from ZCatalog
        ZCatalog.__init__(self, id, title, vocab_id, container)

        self._initialize(container)


    def _initBrainUsage(self):
        # turns brain usage on

        if 1:
                # use brains
                brains = MenuBrain
                if brains and self._catalog._v_brains != brains:
                    self._catalog._v_brains = brains
                    self._catalog.updateBrains()


    def _initialize(self, container=None):
        '''
        initializes the object
        '''

        # More default fields and indexes:
        #self.addColumn('title')
        
        self.addColumn('creator')
        self.addColumn('icon')
        self.addColumn('hasSubItems')
        self.addColumn('getPhysicalPath')
        self.addColumn('getPhysicalURL')
        self.addColumn('getPhysicalParentPath')
        self.addColumn('getPhysicalParentURL')
        self.addColumn('getPhysicalPathLevel')
        self.addColumn('getHiddenFlag')
        self.addColumn('getMenuRootFlag')
        self.addColumn('getMenuRootPhysicalPath')
        self.addColumn('getMenuRootPhysicalURL')
        self.addColumn('getMenuPath')
        self.addColumn('getMenuURL')
        self.addColumn('getMenuLevel')
        self.addColumn('getMenuWeight')
        self.addColumn('getMenuLocation')

        # menuefolder compatibility
        #self.addCollumn('menupath')
        #self.addCollumn('menulevel')

        self.addIndex('creator', 'FieldIndex')
        self.addIndex('getPhysicalURL', 'FieldIndex')
        self.addIndex('getPhysicalParentURL', 'FieldIndex')
        self.addIndex('getHiddenFlag', 'FieldIndex')
        self.addIndex('getMenuRootFlag', 'FieldIndex')
        self.addIndex('getMenuRootPhysicalURL', 'FieldIndex')
        self.addIndex('getMenuURL', 'FieldIndex')
        self.addIndex('getMenuLevel', 'FieldIndex')

Globals.default__class_init__(MenuCatalog)



class MenuItem(Cacheable, CatalogAware, ExtensionClass.Base):
    '''
    menu class
    '''

    # per default off --> set at indexing in catalog
    _isMenuItem=0

    _hidden=0
    _isMenuRoot=0
    _menuWeight=100


    # caching
    __enabled = 1
    _isCacheable = 1


    # default DTML Tree generator
    getDTMLTree = DTMLFile('www/getDTMLTree', globals())

    # standard template
    standardTemplate = PageTemplateFile('www/menuTemplate', globals())
    standardItemView = PageTemplateFile('www/menuItemView', globals())
    standardExample = PageTemplateFile('www/example', globals())    
    
    manage_options = ( (
        {'label': 'getMenu', 'action': 'getMenu',},
        {'label': 'menuPreview', 'action': 'renderItem',},
        {'label': 'reIndexAll', 'action': 'reindex_all',},
        {'label': 'invalidateCacheOfAll', 'action': 'invalidate_all',},
        ) )

    _properties=({'id':'title', 'type': 'string'},
           {'id':'_hidden', 'type': 'int'},
           {'id':'_isMenuRoot', 'type': 'int'},
           {'id':'_menuWeight', 'type': 'int'},
           {'id':'_isMenuItem', 'type': 'int'},
           )


    # helpers
    def _makeItAMenuItem(self, flag):
        # sets _isMenuItem attribute
        setattr(self, '_isMenuItem', int(flag))

    # indizes / meta

    def getPhysicalURL(self):
        return path2url(self.getPhysicalPath())

    def getPhysicalParentPath(self):
        return self.getPhysicalPath()[:-1]
    
    def getPhysicalParentURL(self):
        return path2url(self.getPhysicalParentPath())

    def getMenuPath(self):
        rootPathLevel=len(self.getMenuRootPhysicalPath())
        tmp=self.getPhysicalPath()[rootPathLevel:]
        #print "menupath level", tmp
        return tmp

    def getMenuLevel(self):
        return len(self.getMenuPath())

    def getMenuURL(self):
        return path2url(self.getMenuPath())

    def getVirtualURL(self):
        #print "dudu"
        # This is useful for physical path relative to a VirtualRoot
        try:
            req = self.REQUEST
        except AttributeError:
            #print "no request"
            req = {}
        rpp = req.get('VirtualRootPhysicalPath', ('',))
        spp = self.getPhysicalPath()
        i = 0
        #print "rpp, spp", rpp, spp
        for name in rpp[:len(spp)]:
            #print "name", name, spp[i]
            if spp[i] == name:
                i = i + 1
            else:
                break
            
        path = map(quote, spp[i:])
        #print "path", path, spp[i:]
        return "/"+path2url(path)    
    
    def getMenuLocation(self):
        if not self.getMenuRootFlag():
            try:
                parentLocation=self.aq_parent.getMenuLocation()
            except:
                parentLocation='...'
            location=self.title
            if location == '':
                location=str(self.id)
            return parentLocation+location+'/'
        else:
            return '/'

    def _setHiddenFlag(self, flag):
        self._hidden=int(flag)
        #print "hiddenflag set", self._hidden
        return self._hidden

    def getHiddenFlag(self):
        #print "hiddenflag get", self._hidden
        return self._hidden

    def _setMenuWeight(self, weight):
        self._menuWeight=int(weight)
        return self._menuWeight

    def getMenuWeight(self):
        return self._menuWeight

    def _setMenuRootFlag(self, flag=0):
        self._isMenuRoot=int(flag)
        return self._isMenuRoot

    def getMenuRootFlag(self):
        return self._isMenuRoot

    def getPhysicalPathLevel(self):
        return len(self.getPhysicalPath())

    def getMenuRootPhysicalPath(self):
        ''' gets the menuroots path '''
        menuroot=self.getMenuRootAq()
        return menuroot.getPhysicalPath()

    def getMenuRootPhysicalURL(self):
        return path2url(self.getMenuRootPhysicalPath())

    def hasSubItems(self):
        return len(self.menuLookup(mode=None))


    # helpers

    def reindex_all(self, obj=None):
        """ reindexes myself and all subobjects """

        # TODO: make it private

        if not self.getMenuRootFlag():
            return 'only for menu roots!'

        if obj is None: obj=self
        if hasattr(aq_base(obj), 'index_object'):
            obj.index_object()
        if hasattr(aq_base(obj), 'objectValues'):
            sub=obj.objectValues()
            for item in obj.objectValues():
                self.reindex_all(item)
        return 'done!'


    def invalidate_all(self, obj=None, onlyroot=0):
        """ invalidates cache on all menuItem subobjects """

        # TODO: access control

        if not self.getMenuRootFlag() and onlyroot:
            return

        if obj is None: obj=self
        if hasattr(aq_base(obj), 'ZCacheable_invalidate'):
            #print "invalidating", obj.id
            obj.ZCacheable_invalidate()
        if hasattr(aq_base(obj), 'objectValues'):
            sub=obj.objectValues()
            for item in obj.objectValues():
                if getattr(item, '_isMenuItem', None):
                    self.invalidate_all(item)
        return 'done!'


        if obj is None: obj=self
        if hasattr(aq_base(obj), 'index_object'):
            obj.index_object()
        if hasattr(aq_base(obj), 'objectValues'):
            sub=obj.objectValues()
            for item in obj.objectValues():
                self.reindex_all(item)
        return 'done!'


    def isSelected(self, selectedMenuURL):
        ''' returns 1 or 0 if this brain is part of the selected menu or not '''

        selectedMenuPath=url2path(selectedMenuURL)

        if self.getMenuPath() == selectedMenuPath[:len(self.getMenuPath())]:
            return 1
        else:
            return 0


    def getMenuTree(self, resultlist):
        # returns only brains which are in my own menutree
       
        r=[]
        myPath=self.getPhysicalPath()
        #print "myPath", myPath
        for result in resultlist:
            rPath=result.getPhysicalPath
            if callable(rPath):
                rPath=rPath()
            #print "rPath", rPath
            if myPath[:len(rPath)] == rPath:
                #print "is in my tree"
                r.append(result)

        return r


    def sortMenuLevel(self, brainList, sorter=None, sortMethod=None):
        # returns the brainList sorted by custom sort(if set) -> weight -> alphanurmerical

        if not sorter or not callable(sorter):
            sorter=MenuSorter(sortMethod)

        return sorter.sortMenuLevel(brainList)


    def getMenuRootAq(self):
        # returns my menuroot object per aquisition

        if self.getMenuRootFlag():
            return self

        menuRoot=None
        if hasattr(self, 'aq_chain'):
            for ob in self.aq_chain:
                if hasattr(ob, 'getMenuRootFlag'):
                    if ob.getMenuRootFlag():
                        menuRoot=ob
                        break

        if not menuRoot:
            return self

        return menuRoot

    def getMenuRoot(self):
        # returns my menuroot brain

        if self.getMenuRootFlag():
            menuRoot=self.getSelf()
            if menuRoot:
                return menuRoot
            else:
                return self

        terms={'getMenuRootFlag': 1}
        list = self.getMenuTree(self.searchCatalog(terms))

        if not len(list):
            return self.getMenuRootAq()

        if len(list) == 1:
            return list[0]
        else:
            r=list[0]
            myLevel=self.getPhysicalPathLevel()
            deepest=0
            for result in list:
                current=result.getPhysicalPathLevel
                if current > deepest:
                    deepest=current
                    r=result

        #print "got result", r
        return r


    def searchCatalog(self, terms={}, used=None):
        ''' catalog search method '''

        #print "searching in catalog", terms


        catalog=self.getMenuCatalog()
        r=catalog.searchResults(terms, used)

        #print "s result", r
        return r

    def queryCatalog(self, query_expr, namespace={}):
        ''' query a catalog (requires CatalogQuery Product) '''

        #print "query in catalog"

        try:
            catalog=self.getMenuCatalog()
            r = catalog.query(query_expr, namespace)

            #print "q result", r
            return r
        except:
            #print "no CatalogQuery installed?"
            return []

    def getMenuCatalog(self):
        # get the catalog

        catalog=getattr(self,'menuCatalog',None)
        if catalog:
            catalog._initBrainUsage()

        return catalog


    def getSelf(self, noRetry=0):
        ''' returns my own catalog brain '''

        terms={'getPhysicalURL': self.getPhysicalURL()}
        list = self.searchCatalog(terms)
        if len(list):
            return list[0]
        else:
            if noRetry:
                #print "giving up to reindex myself", self.id
                return None
            else:
                #print "not found myself in catalog! reindexing", self.id
                self.index_object()
                return self.getSelf(noRetry=1)


    def findTemplate(self, kw={}):
        # returns a valid callable pagetemplate instance

        # to make sure the catalog is ok
        catalog=self.getMenuCatalog()

        template=kw.get('template', None)

        if callable(template):
            return template

        if type(template) == types.StringType:
            if not len(template):
                template=self.standardItemView

        if not callable(template):
            template = getattr(self, 'menuTemplate_'+str(template), None)

        if not template:
            template=self.standardItemView

        #print "using template", template
        return template


    def _correctContext(self, args, kw):
        # corrects the context for double aq
        # THIS IS A HACK
        # TODO: prove that this works everytime
        # TOOD: prevend inheritance of _isMenuItem
        
        #print "correct start", self.REQUEST.steps
        
        steps=join(self.REQUEST.steps, '/')
        ob = self.unrestrictedTraverse(steps)
        name=None
        test=None
        try:
            ob=ob.aq_parent
            test=ob._isMenuItem
        except:
            name=self.REQUEST.steps[-1]
        
        if not test:
            steps=join(self.REQUEST.steps[:-1], '/')
            ob = self.unrestrictedTraverse(steps)
            
            if not getattr(ob, '_isMenuItem', 0):
                ob=ob.aq_parent
                
            if ob and ob != self and ob._isMenuItem:
                method=getattr(ob, name)
                return apply(method, args, kw)
        
        #print "richtiger context"
        raise "correct context"
        
    
    def renderMenu(self, *args, **kw):
        ''' renders the menu and returns text/html '''

        
        try: return self._correctContext(args, kw)
        except: pass
        
        
        template=self.findTemplate(kw)
        menuRoot=self.getMenuRoot()
        
        # get kw params
        showParent=kw.get('showParent', 1)
        showHidden=kw.get('showHidden', 0)
        sorter=kw.get('sorter', None)
        selectedMenuURL=kw.get('selectedMenuURL', self.getMenuURL)
        startlevel=kw.get('startlevel', None)
        
        # set maxlevel (actual+1)
        maxlevel=self.getMenuLevel()+1
        
        # check startlevel param
        if startlevel != None:
            results = self.menuLookup(mode=int(startlevel)-1)
            results = self.getMenuTree(results)
            if len(results): menuRoot=results[0]
            else:
                menuRoot=self.getSelf()
                maxlevel=self.getMenuLevel()
            
        if self.ZCacheable_isCachingEnabled():

            keys={}
            keys.update(kw)
            keys['menuRootURL']=menuRoot.getPhysicalURL

            view_name='renderMenu'
            result=self.ZCacheable_get(view_name=view_name, keywords=keys)
            if result:
                #print "returning cached renderMenu"
                return result

        data = self.renderItem(parent=menuRoot, template=template, maxlevel=maxlevel, selectedMenuURL=selectedMenuURL, showParent=showParent, showHidden=showHidden, sorter=sorter)

        if self.ZCacheable_isCachingEnabled():
            self.ZCacheable_set(data, view_name=view_name, keywords=keys)

        return data


    def renderSiteMap(self, *args, **kw):
        ''' renders the complete menu with all trees expanded (siteMap) '''

        
        try: return self._correctContext(args, kw)
        except: pass        
        
        template=self.findTemplate(kw)

        # get kw params
        showParent=kw.get('showParent', 1)
        showHidden=kw.get('showHidden', 0)
        sorter=kw.get('sorter', None)
        flag=kw.get('flag', None)

        if not flag:
            menuRoot=self.getMenuRoot()
        else:
            menuRoot=self.getSelf()

        if self.ZCacheable_isCachingEnabled():

            keys={}
            keys.update(kw)
            keys['menuRootURL']=menuRoot.getPhysicalURL

            view_name='renderSiteMap'
            result=self.ZCacheable_get(view_name=view_name, keywords=keys)
            if result:
                #print "returning cached renderSiteMap"
                return result

        data = self.renderItem(parent=menuRoot, template=template, selectedMenuURL=menuRoot.getMenuURL, flag=None, showParent=showParent, showHidden=showHidden, sorter=sorter)

        if self.ZCacheable_isCachingEnabled():
            self.ZCacheable_set(data, view_name=view_name, keywords=keys)

        return data


    def renderSubLevel(self, *args, **kw):
        ''' renders one single menulevel (my sublevel)'''

        try: return self._correctContext(args, kw)
        except: pass        
        
        template=self.findTemplate(kw)

        # get kw params
        parent = kw.get('parent', self.getSelf())
        showHidden=kw.get('showHidden', 0)
        flag=kw.get('flag', 'myTree')
        sorter=kw.get('sorter', None)
        selectedMenuURL=kw.get('selectedMenuURL', None)

        if self.ZCacheable_isCachingEnabled():

            keys={}
            keys.update(kw)
            keys['parent']=parent.getPhysicalURL

            view_name='renderSubLevel'
            result=self.ZCacheable_get(view_name=view_name, keywords=keys)
            if result:
                #print "returning cached renderSubLevel"
                return result

        data = self.renderItem(parent=parent, template=template, selectedMenuURL=selectedMenuURL, flag=flag, showParent=0, maxlevel=parent.getMenuLevel+1, showHidden=showHidden, sorter=sorter)

        if self.ZCacheable_isCachingEnabled():
            self.ZCacheable_set(data, view_name=view_name, keywords=keys)

        return data


    def renderOwnLevel(self, *args, **kw):
        ''' renders one single menulevel (my own level)'''

        try: return self._correctContext(args, kw)
        except: pass        
        
        # TODO: caching for this method

        template=self.findTemplate(kw)

        # get kw params
        parent = kw.get('parent', self.getSelf())
        showHidden=kw.get('showHidden', 0)
        sorter=kw.get('sorter', None)
        selectedMenuURL=kw.get('selectedMenuURL', None)

        return self.renderItem(parent=parent, template=template, selectedMenuURL=selectedMenuURL, showParent=1, maxlevel=parent.getMenuLevel, showHidden=showHidden, sorter=sorter)
    

    def renderMenuLevel(self, *args, **kw):
        ''' renders a single defined menu level '''

        #print "1"
        try: return self._correctContext(args, kw)
        except: pass

        #print "2"
        
        template=self.findTemplate(kw)

        # get kw params
        level=int(kw.get('level', 1))
        showHidden=kw.get('showHidden', 0)
        flag=kw.get('flag', 'myTree')
        sorter=kw.get('sorter', None)
        selectedMenuURL=kw.get('selectedMenuURL', None)

        if level > 0:
            # get this levels brains with parents help
            level=level-1
            renderMethod=self.renderSubLevel
        else:
            # render the roots level
            renderMethod=self.renderOwnLevel

        results = self.menuLookup(mode=level)
        
        if flag == 'myTree':
            results=self.getMenuTree(results)
        
        html=''
        #print "3"
        for ob in results:
            #print ob.title
            html += renderMethod(parent=ob, flag=flag, template=template, selectedMenuURL=selectedMenuURL, showHidden=showHidden, sorter=sorter)
            #print "4"
        
        if not len(html):
            return ''

        #print "5"
        return html


    def renderItem(self, parent=None, template=None, maxlevel=-1, selectedMenuURL=None, flag='myTree', showParent=1, showHidden=0, sorter=None, sortMethod=None):
        ''' renders one single menuitem and returns text/html'''

        if not template or not callable(template):
            template=self.findTemplate()

        if not parent:
            parent=self.getSelf()

        if not selectedMenuURL:
            selectedMenuURL=parent.getMenuURL

        #print "rendering item", parent.title

        showSubLevel=1
        children=[]

        if maxlevel != -1:
            # a max level deepness was set
            if maxlevel <= parent.getMenuLevel:
                showSubLevel=0

        if flag == 'myTree':

            # filter out parallel levels not in my tree
            # TODO: find better way to filter out parallel menu items
            # TODO: put it before catalog search to increase performance
            test=[parent]
            test=self.getMenuTree(test)
            if not len(test):
                showSubLevel=0

        if showSubLevel:

            terms={}
            #terms['getMenuLevel'] = parent.getMenuLevel+1 # no subroots are shown when this is set!
            terms['getPhysicalParentURL'] = parent.getPhysicalURL

            if not showHidden:
                terms['getHiddenFlag']=0

            children = self.searchCatalog(terms)

            # get sorting helper instance
            if not sorter or sortMethod:
                #print "creating new sorter instance"
                sorter=MenuSorter(sortMethod)

            # sort children
            children=self.sortMenuLevel(children, sorter)

        #print parent.id, children
        return template(parent=parent, children=children, maxlevel=maxlevel, selectedMenuURL=selectedMenuURL, flag=flag, showParent=showParent, showHidden=showHidden)


    # menu generation
    def menuLookup(self, mode=None):
        """ returns menu objects helper method"""

        result=[]
        terms={}

        #print "modetype", type(mode)

        #terms['sort_on']='getMenuLevel'

        # failsave mode -> find subobjects without catalog
        if mode == None:
            for object in self.objectValues():
                test =  getattr(object, '_isMenuItem', None)
                if test:
                    result.append(object)

        elif mode == 'thisLevel':
            terms['getMenuLevel'] = self.getMenuLevel()
            terms['getMenuRootPhysicalURL'] = self.getMenuRootPhysicalURL()

            result = self.searchCatalog(terms)

        elif mode == 'subLevel':
            terms['getMenuLevel'] = self.getMenuLevel()+1
            terms['getPhysicalParentURL'] = self.getPhysicalURL()

            result = self.searchCatalog(terms)

        elif mode == 'complete':
            terms['getMenuRootPhysicalURL'] = self.getMenuRootPhysicalURL()

            result = self.searchCatalog(terms)

        elif type(mode) == types.IntType:

            terms['getMenuLevel'] = mode
            terms['getMenuRootPhysicalURL'] = self.getMenuRootPhysicalURL()
            
            result = self.searchCatalog(terms)


        elif type(mode) == types.ListType:
            if len(mode) == 2:

                # TODO: fix query language
                levelStart=mode[0]
                levelEnd=mode[1]
                result = self.queryCatalog('getMenuLevel between [levelStart, levelEnd] or getMenuLevel == levelStart or getMenuLevel ==levelEnd')

        '''
        result=list(result)
        user = getSecurityManager().getUser()
        for brain in result:
            if mode == None: obj=brain
            else: obj=brain.getObject()
            roles = rolesForPermissionOn('View', obj)
            if type(roles) is type(''):
                roles=[roles]

            if not user.allowed( obj, roles ):
                #print "removing", brain.title
                result.remove(brain)

        #print "result", result
        '''
        return result


    # property things
    def manage_editProperties(self, REQUEST):
        """Edit object properties via the web.
        The purpose of this method is to change all property values,
        even those not listed in REQUEST; otherwise checkboxes that
        get turned off will be ignored.  Use manage_changeProperties()
        instead for most situations.
        """
        for prop in self._propertyMap():
            name=prop['id']
            if 'w' in prop.get('mode', 'wd'):
                value=REQUEST.get(name, '')
                self._updateProperty(name, value)

        self.reindex_object()

        if REQUEST:
            message="Saved changes."
            return self.manage_propertiesForm(self,REQUEST,
                                              manage_tabs_message=message)


    # catalog (un)indexing

    def index_object(self):
        ''' A common method to allow findables to index themselves '''

        #print "indexing object", self.getPath()

        self._makeItAMenuItem(1)
        catalog=self.getMenuCatalog()
        catalog.catalog_object(self, self.getPath())

        try:
            catalog.refreshCatalog()
        except:
            pass
        self.ZCacheable_invalidate()

        try:
            menuRoot=self.getMenuRootAq()
            menuRoot.invalidate_all(onlyroot=1)
        except:
            print "error while invalidating cache"

    def unindex_object(self):
        ''' A common method to allow findabled to unindex themselves '''

        #print "unindexing object", self.getPath()

        catalog=self.getMenuCatalog()
        catalog.uncatalog_object(self.getPath())

        if not self.getMenuRootFlag():
            self._makeItAMenuItem(0)

        try:
            catalog.refreshCatalog()
        except:
            pass
        self.ZCacheable_invalidate()

        try:
            menuRoot=self.getMenuRootAq()
            menuRoot.invalidate_all(onlyroot=1)
        except:
            print "error while invalidating cache"


    def _refresh_catalog(self):
        ''' A simple method which refreshes the menu catalog
           call it to make changes appear in the menu
        '''

        catalog=self.getMenuCatalog()
        try:
            catalog.refreshCatalog()
        except:
            pass
        self.ZCacheable_invalidate()


Globals.default__class_init__(MenuItem)


class MenuFolder(MenuItem, Folder):
    '''
    sample class of an easy menu item
    '''

    meta_type = 'MenuFolder'

    manage_options=Folder.manage_options + MenuItem.manage_options

    def __init__(self, id, title='', hidden=0, isRoot=None, menuWeight=None):

        self.id=str(id)
        self.title=title

        if hidden:
            self._setHiddenFlag(1)

        if menuWeight:
            self._setMenuWeight(int(menuWeight))

        if isRoot:

            self._setMenuRootFlag(1)
            id='menuCatalog'
            ob=MenuCatalog(id,'',None,self)
            ob.id=id
            self._setObject( id, ob )


Globals.default__class_init__(MenuFolder)


manage_addMenuFolderForm = DTMLFile( 'www/menuFolderAdd', globals() )

def manage_addMenuFolder( self, id, title='', hidden=None, isRoot=None, menuWeight=100, REQUEST=None ):
    '''Add a MenuFolder instance
    '''

    id=str(id)
    ob = MenuFolder(id, title, hidden, isRoot, menuWeight)

    self._setObject( id, ob )
    ob = self._getOb( id )

    checkPermission = getSecurityManager().checkPermission

    if isRoot:
        if not checkPermission( 'Add Documents, Images, and Files', ob ):
            raise 'Unauthorized', ('You are not authorized to add Documents.')

        filename=ob.standardExample.filename
        from Products.PageTemplates import ZopePageTemplate
        fp=open(filename, "rb", 1)
        ZopePageTemplate.manage_addPageTemplate( ob, id='index_html', title='example index page with navigation', text=fp )
        fp.close()


    if REQUEST is not None:
        return self.manage_main( self, REQUEST, update_menu=1 )

